/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring;

import gz.cyberbrain.cyberhotel.model.cash.dao.CashDao;
import gz.cyberbrain.cyberhotel.model.cash.to.CashTo;
import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.delegate.InvoicingFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.ClosedPaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.IssuedInvoiceException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptableOweException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.ClosedCashException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.DeleteAPaymentAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.DeleteAnInvoiceAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.InsertAPaymentAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.InsertAnInvoiceAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.PayedInvoiceException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.UpdateACashAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.UpdateAPaymentAction;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.UpdateAnInvoiceAction;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.payment.dao.PaymentDao;
import gz.cyberbrain.cyberhotel.model.payment.to.PaymentTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.util.Collection;

public class InvoicingSpringFacadeDelegate implements InvoicingFacadeDelegate {

	private static final long serialVersionUID = 1L;

	private InvoiceDao invoiceDao;
	private OweDao oweDao;
	private ClientDao agencyDao;
	private ClientDao companyDao;
	private ClientDao particularDao;
	private PaymentDao paymentDao;
	private CashDao cashDao;

	public void setInvoiceDao(InvoiceDao invoiceDao) {
		this.invoiceDao = invoiceDao;
	}

	public void setOweDao(OweDao oweDao) {
		this.oweDao = oweDao;
	}

	public void setAgencyDao(ClientDao agencyDao) {
		this.agencyDao = agencyDao;
	}

	public void setCompanyDao(ClientDao companyDao) {
		this.companyDao = companyDao;
	}

	public void setParticularDao(ClientDao particularDao) {
		this.particularDao = particularDao;
	}

	public void setPaymentDao(PaymentDao paymentDao) {
		this.paymentDao = paymentDao;
	}
	
	public void setCashDao(CashDao cashDao) {
		this.cashDao = cashDao;
	}

	public InvoiceTo insertAnInvoice(InvoiceTo invoiceTo, Collection oweNumbers) throws UnaceptablePayerException, UnaceptableOweException, InstanceNotFoundException, InternalErrorException {
		InsertAnInvoiceAction action = new InsertAnInvoiceAction(invoiceDao, oweDao, agencyDao, companyDao, particularDao, invoiceTo, oweNumbers);
		return action.execute();
	}

	public InvoiceTo findAnInvoice(Long invoiceNumber, String invoiceSerial) throws InstanceNotFoundException, InternalErrorException {
		return invoiceDao.find(invoiceNumber, invoiceSerial);
	}

	public void updateAnInvoice(InvoiceTo invoiceTo, Collection oweNumbers) throws UnaceptablePayerException, UnaceptableOweException, InstanceNotFoundException, InternalErrorException {
		UpdateAnInvoiceAction action = new UpdateAnInvoiceAction(invoiceDao, oweDao, agencyDao, companyDao, particularDao, invoiceTo, oweNumbers);
		action.execute();
	}

	public void deleteAnInvoice(InvoiceTo invoiceTo) throws PayedInvoiceException, InstanceNotFoundException, InternalErrorException {
		DeleteAnInvoiceAction action = new DeleteAnInvoiceAction(invoiceDao, oweDao, invoiceTo);
		action.execute();
	}

	public Collection showInvoices() throws InternalErrorException {		
		return invoiceDao.findAll();		
	}

	public ClientTo findAPayer(Long clientId) throws InstanceNotFoundException, InternalErrorException {

		ClientTo clientTo = null;

		try {

			clientTo = particularDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		try {

			clientTo = agencyDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		try {

			clientTo = companyDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		if (clientTo != null) return clientTo;
		else throw new InstanceNotFoundException(clientTo, "Payer not found.");

	}

	public Collection findInvoiceOwes(Long invoiceNumber, String invoiceSerial) throws InternalErrorException {
		return invoiceDao.findInvoiceOwes(invoiceNumber, invoiceSerial);		
	}	

	public Collection findOutstandingInvoices() throws InternalErrorException {
		return invoiceDao.findOutstanding();
	}

	public Collection findIssuedInvoices() throws InternalErrorException {
		return invoiceDao.findIssued();
	}

	public Collection showPayments() throws InternalErrorException {
		return paymentDao.findAll();
	}

	public PaymentTo insertAPayment(PaymentTo paymentTo, boolean issue) 
	throws UnaceptableInvoiceException, IssuedInvoiceException, InstanceNotFoundException, InternalErrorException {

		InsertAPaymentAction action = new InsertAPaymentAction(invoiceDao, paymentDao, paymentTo, issue);
		return action.execute();

	}

	public PaymentTo findAPayment(Long reference) throws InstanceNotFoundException, InternalErrorException {
		return paymentDao.find(reference);
	}

	public void updateAPayment(PaymentTo paymentTo) 
	throws ClosedPaymentException, UnaceptableInvoiceException, IssuedInvoiceException,  InstanceNotFoundException, InternalErrorException {

		UpdateAPaymentAction action = new UpdateAPaymentAction(invoiceDao, paymentDao, paymentTo);
		action.execute();

	}

	public void deleteAPayment(PaymentTo paymentTo) throws InstanceNotFoundException, InternalErrorException {

		DeleteAPaymentAction action = new DeleteAPaymentAction(invoiceDao, paymentDao, paymentTo);
		action.execute();

	}

	public CashTo insertACash(CashTo cashTo) throws InstanceNotFoundException, InternalErrorException {
		return cashDao.create(cashTo);
	}
	
	public CashTo findACash(Long cashNumber) throws InstanceNotFoundException, InternalErrorException {
		return cashDao.find(cashNumber);
	}

	public void updateACash(CashTo cashTo, Collection payments, boolean closement) throws ClosedCashException, UnaceptablePaymentException, ClosedPaymentException, InstanceNotFoundException, InternalErrorException {
		UpdateACashAction action = new UpdateACashAction(cashDao, paymentDao, cashTo, payments, closement);
		action.execute();
		
	}
	
	public Collection showCashes() throws InternalErrorException {
		return cashDao.findAll();
	}

	public Collection findOpenedPayments() throws InternalErrorException {
		return paymentDao.findOpened();
	}

}
