/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.dao.ReservationDao;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnregisteredReservationException;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.util.Collection;
import java.util.Iterator;

public class RemoveACheckInAction {

	private ReservationTo reservationTo;
	private ReservationDao reservationDao;
	private OweDao oweDao;
	private RoomDao roomDao;

	public RemoveACheckInAction(ReservationTo reservationTo, ReservationDao reservationDao, OweDao oweDao, RoomDao roomDao) {		
		this.reservationTo = reservationTo;
		this.reservationDao = reservationDao;
		this.oweDao = oweDao;
		this.roomDao = roomDao;
	}
	
	public void execute() 
	throws FinishedReservationException, CancelledReservationException, UnregisteredReservationException, 
	InstanceNotFoundException, InternalErrorException {
		
		if (reservationTo.getReservationStatus().equals("Pendente")) 
			throw new UnregisteredReservationException(reservationTo);
		
		if (reservationTo.getReservationStatus().equals("Anulada")) 
			throw new CancelledReservationException(reservationTo);
		
		if (reservationTo.getReservationStatus().equals("Finalizada"))
			throw new FinishedReservationException(reservationTo);
		
		Collection owes = reservationDao.findReservationOwes(reservationTo.getReservationId());
		
		Iterator iter = owes.iterator();
		
		for (int i=0; iter.hasNext();i++) {			
			oweDao.remove(oweDao.find(new Long((Short) iter.next())));
		}
		
		//RequestDao requestDao = SpringDaoFactory.getRequestDao();
		
		Collection requests = reservationDao.findRequests(reservationTo.getReservationId());
		
		iter = requests.iterator();
		
		RoomTo updatedRoom;
		
		for (int i=0; iter.hasNext();i++) {
			updatedRoom = roomDao.find(((RequestTo) iter.next()).getRoom());
			updatedRoom.setOccupancyStatus("Libre");
			roomDao.update(updatedRoom);
		}
		
		/*for (int i=0; iter.hasNext();i++) {
			requestDao.remove((RequestTo) iter.next());
		}
		
		reservationDao.remove(reservationTo);*/
		
		reservationTo.setReservationStatus("Pendente");
		reservationDao.update(reservationTo);
		
		
	}
	
}
