/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.reception;

import gz.cyberbrain.cyberhotel.controller.EmployeeListAdapter;
import gz.cyberbrain.cyberhotel.controller.WashController;
import gz.cyberbrain.cyberhotel.view.swing.edit.RoomWindow;
import gz.cyberbrain.cyberhotel.view.swing.start.MainWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import javax.swing.JPanel;
import javax.swing.JComboBox;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DateFormat;
import java.util.Calendar;

import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JButton;

import org.jdesktop.swingx.JXDatePicker;

import javax.swing.JLabel;

public class WashWindow extends JDialog {

	private static final long serialVersionUID = 1L;
	private static WashWindow washWindow = null;
	private JPanel jContentPane = null;
	private JXDatePicker dateComboBox = null;
	private JComboBox employeeComboBox = null;
	private JCheckBox inCheckBox = null;
	private JCheckBox outCheckBox = null;
	private JCheckBox towelsCheckBox = null;
	private JCheckBox sheetsCheckBox = null;
	private JCheckBox disinfectionCheckBox = null;
	private JCheckBox otherCheckBox = null;
	private JScrollPane obsScrollPane = null;
	private JTextArea obsTextArea = null;
	private JButton acceptButton = null;
	private JButton cancelButton = null;
	private JLabel obsLabel = null;
	private JLabel dateLabel = null;
	private JLabel employeeLabel = null;
	private String roomCode = null;
	private Long employeeId = null;

	/**
	 * This is the xxx default constructor
	 */
	private WashWindow() {
		super(MainWindow.getInstance());
		initialize();
	}

	public static WashWindow getInstance() {
		if(washWindow == null) washWindow = new WashWindow();
		return washWindow;
	}	
	
	public Long getEmployeeId() {
		return employeeId;
	}

	public void setEmployeeId(Long employeeId) {
		this.employeeId = employeeId;
	}

	public String getRoomCode() {
		return roomCode;
	}

	public void setRoomCode(String roomCode) {
		this.roomCode = roomCode;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(507, 283);
		this.setTitle(Locale.get("WashWindow.title"));
		this.setContentPane(getJContentPane());
		this.setModal(true);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent we) {
		       dispose();
		    	washWindow = null;
		    }
		});
			
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			employeeLabel = new JLabel();
			employeeLabel.setBounds(new Rectangle(154, 15, 53, 16));
			employeeLabel.setText(Locale.get("WashWindow.employeeLabel"));
			dateLabel = new JLabel();
			dateLabel.setBounds(new Rectangle(6, 15, 38, 16));
			dateLabel.setText(Locale.get("WashWindow.dateLabel"));
			obsLabel = new JLabel();
			obsLabel.setBounds(new Rectangle(20, 115, 79, 16));
			obsLabel.setText(Locale.get("WashWindow.obsLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getDateComboBox(), null);
			jContentPane.add(getEmployeeComboBox(), null);
			jContentPane.add(getInCheckBox(), null);
			jContentPane.add(getOutCheckBox(), null);
			jContentPane.add(getTowelsCheckBox(), null);
			jContentPane.add(getSheetsCheckBox(), null);
			jContentPane.add(getDisinfectionCheckBox(), null);
			jContentPane.add(getOtherCheckBox(), null);
			jContentPane.add(getObsScrollPane(), null);
			jContentPane.add(getAcceptButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(obsLabel, null);
			jContentPane.add(dateLabel, null);
			jContentPane.add(employeeLabel, null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes dateComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JXDatePicker getDateComboBox() {
		if (dateComboBox == null) {
			dateComboBox = new JXDatePicker();
			dateComboBox.setBounds(new Rectangle(49, 13, 98, 19));
			DateFormat []df = {DateFormat.getDateInstance(DateFormat.SHORT)}; 
			dateComboBox.setFormats(df);
		}
		return dateComboBox;
	}

	/**
	 * This method initializes employeeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getEmployeeComboBox() {
		if (employeeComboBox == null) {
			employeeComboBox = new JComboBox(new EmployeeListAdapter().load());
			employeeComboBox.setBounds(new Rectangle(213, 13, 281, 19));
		}
		return employeeComboBox;
	}

	/**
	 * This method initializes inCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getInCheckBox() {
		if (inCheckBox == null) {
			inCheckBox = new JCheckBox();
			inCheckBox.setBounds(new Rectangle(97, 54, 74, 21));
			inCheckBox.setText(Locale.get("WashWindow.inCheckBox"));
		}
		return inCheckBox;
	}

	/**
	 * This method initializes outCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getOutCheckBox() {
		if (outCheckBox == null) {
			outCheckBox = new JCheckBox();
			outCheckBox.setBounds(new Rectangle(97, 87, 74, 21));
			outCheckBox.setText(Locale.get("WashWindow.outCheckBox"));
		}
		return outCheckBox;
	}

	/**
	 * This method initializes towelsCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getTowelsCheckBox() {
		if (towelsCheckBox == null) {
			towelsCheckBox = new JCheckBox();
			towelsCheckBox.setBounds(new Rectangle(215, 54, 83, 21));
			towelsCheckBox.setText(Locale.get("WashWindow.towelsCheckBox"));
		}
		return towelsCheckBox;
	}

	/**
	 * This method initializes sheetsCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getSheetsCheckBox() {
		if (sheetsCheckBox == null) {
			sheetsCheckBox = new JCheckBox();
			sheetsCheckBox.setBounds(new Rectangle(215, 87, 83, 21));
			sheetsCheckBox.setText(Locale.get("WashWindow.sheetsCheckBox"));
		}
		return sheetsCheckBox;
	}

	/**
	 * This method initializes disinfectionCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getDisinfectionCheckBox() {
		if (disinfectionCheckBox == null) {
			disinfectionCheckBox = new JCheckBox();
			disinfectionCheckBox.setBounds(new Rectangle(333, 54, 102, 21));
			disinfectionCheckBox.setText(Locale.get("WashWindow.disinfectionCheckBox"));
		}
		return disinfectionCheckBox;
	}

	/**
	 * This method initializes otherCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getOtherCheckBox() {
		if (otherCheckBox == null) {
			otherCheckBox = new JCheckBox();
			otherCheckBox.setBounds(new Rectangle(333, 87, 74, 21));
			otherCheckBox.setText(Locale.get("WashWindow.otherCheckBox"));
		}
		return otherCheckBox;
	}

	/**
	 * This method initializes obsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getObsScrollPane() {
		if (obsScrollPane == null) {
			obsScrollPane = new JScrollPane();
			obsScrollPane.setBounds(new Rectangle(19, 141, 466, 68));
			obsScrollPane.setViewportView(getObsTextArea());
		}
		return obsScrollPane;
	}

	/**
	 * This method initializes obsTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getObsTextArea() {
		if (obsTextArea == null) {
			obsTextArea = new JTextArea();
		}
		return obsTextArea;
	}

	/**
	 * This method initializes acceptButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getAcceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton();
			acceptButton.setBounds(new Rectangle(148, 217, 78, 15));
			acceptButton.setText(Locale.get("WashWindow.acceptButton"));
			acceptButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					

					Calendar washDate = Calendar.getInstance();
					washDate.setTime(dateComboBox.getDate());				
					String room = getRoomCode();
					Long employee = (Long) ((TextItem) getEmployeeComboBox().getSelectedItem()).getId();
					String observations = obsTextArea.getText();
					
						WashController.createWash(washDate, inCheckBox.isSelected(),  outCheckBox.isSelected(), 
								towelsCheckBox.isSelected(), sheetsCheckBox.isSelected(), disinfectionCheckBox.isSelected(),
								otherCheckBox.isSelected(), room, employee, observations);
				
						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(RoomWindow.getInstance(),
								Locale.get("JOptionPane.WashWindow.acceptButton.message"),
								Locale.get("JOptionPane.WashWindow.acceptButton.title"),
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {							
							dispose();	                 
							washWindow = null;
						}

				}

			});
		}
		return acceptButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(271, 217, 78, 15));
			cancelButton.setText(Locale.get("WashWindow.cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					

				dispose();
				washWindow = null;
				
				}

			});
		}
		return cancelButton;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
