/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.util.gui;

import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JFormattedTextField;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.Component;
import java.text.NumberFormat;
import java.text.ParseException;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

/**
 * Implements a cell editor that uses a formatted text field
 * to edit number values.
 */
public class NumberEditor extends DefaultCellEditor {

	private static final long serialVersionUID = 1L;
	JFormattedTextField ftf;
	NumberFormat numberFormat;
	private boolean DEBUG = false;

	public NumberEditor() {

		super(new JFormattedTextField());
		ftf = (JFormattedTextField) getComponent();

		//Set up the editor for the number cells.

		ftf.setFormatterFactory(new DefaultFormatterFactory(new NumberFormatter(NumberFormat.getNumberInstance())));
		ftf.setHorizontalAlignment(JTextField.TRAILING);
		ftf.setFocusLostBehavior(JFormattedTextField.PERSIST);

		ftf.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyTyped(KeyEvent e) {

				if (e.getKeyChar() == KeyEvent.VK_PERIOD) e.setKeyChar(',');			    	  			        
				if (!Character.isDigit(e.getKeyChar()) && e.getKeyChar() != KeyEvent.VK_COMMA) e.consume();
				if ((e.getKeyChar() == KeyEvent.VK_COMMA) && ftf.getText().contains(",")) e.consume();
				
			}
		});

		//React when the user presses Enter while the editor is
		//active.  (Tab is handled as specified by
		//JFormattedTextField's focusLostBehavior property.)
		ftf.getInputMap().put(KeyStroke.getKeyStroke(
				KeyEvent.VK_ENTER, 0),
		"check");

		ftf.getActionMap().put("check", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (!ftf.isEditValid()) { //The text is invalid.
				} else try {              //The text is valid,
					ftf.commitEdit();     //so use it.
					ftf.postActionEvent(); //stop editing
				} catch (java.text.ParseException exc) { }
			}
		});

	}

	//Override to invoke setValue on the formatted text field.
	public Component getTableCellEditorComponent(JTable table,
			Object value, boolean isSelected,
			int row, int column) {
		JFormattedTextField ftf =
			(JFormattedTextField)super.getTableCellEditorComponent(
					table, value, isSelected, row, column);
		ftf.setValue(value);
		return ftf;
	}

	//Override to ensure that the value remains a number.
	public Object getCellEditorValue() {
		JFormattedTextField ftf = (JFormattedTextField)getComponent();
		Object o = ftf.getValue();
		if (o instanceof Number) {
			return o;
		} else {
			if (DEBUG) {
				System.out.println("getCellEditorValue: o isn't a Number");
			}
			try {
				return numberFormat.parseObject(o.toString());
			} catch (ParseException exc) {
				System.err.println("getCellEditorValue: can't parse o: " + o);
				return null;
			}
		}
	}

	//Override to check whether the edit is valid,
	//setting the value if it is and complaining if
	//it isn't.  If it's OK for the editor to go
	//away, we need to invoke the superclass's version 
	//of this method so that everything gets cleaned up.
	public boolean stopCellEditing() {
		JFormattedTextField ftf = (JFormattedTextField)getComponent();
		if (ftf.isEditValid()) {
			try {
				ftf.commitEdit();
			} catch (java.text.ParseException exc) { }

		} else { //text is invalid	
		}		
		return super.stopCellEditing();
	}
}
